@@ -1,6 +1,7 @@
 use crate::{
     activity::{notify_activity, wait_activity},
     cache::ClientCache,
+    fd_semaphore::acquire_fd,
     stats::global_sosistab_stats,
     tunman::{TunnelManager, TunnelState},
     AuthOpt, CommonOpt,
@@ -8,13 +9,14 @@ use crate::{
 use crate::{china, plots::stat_derive};
 use anyhow::Context;
 use async_compat::Compat;
-use async_net::IpAddr;
+use async_net::{IpAddr, TcpStream};
 use china::is_chinese_ip;
 use http_types::{Method, Request, Url};
 use once_cell::sync::Lazy;
 use psl::Psl;
 use smol::prelude::*;
-use socksv5::v4;
+
+use tap::Tap;
 
 use std::{
     collections::BTreeMap, net::Ipv4Addr, net::SocketAddr, net::SocketAddrV4, path::PathBuf,
@@ -46,7 +48,7 @@ pub struct ConnectOpt {
     /// Number of local UDP ports to use per session. This works around situations where unlucky ECMP routing sends flows down a congested path even when other paths exist, by "averaging out" all the possible routes.
     pub udp_shard_count: usize,
 
-    #[structopt(long, default_value = "60")]
+    #[structopt(long, default_value = "3")]
     /// Lifetime of a single UDP port. Geph will switch to a different port within this many seconds.
     pub udp_shard_lifetime: u64,
 
@@ -169,6 +171,10 @@ pub async fn main_connect(opt: ConnectOpt) -> anyhow::Result<()> {
                                 .await?;
                         }
                         log::error!("***** ABNORMAL RESTART (status code {}) *****", scode);
+
+                        // Attempt to kill any possible other Geph
+                        let res = kill_existing_geph(opt.stats_listen).await;
+                        log::debug!("kill resulted in {:?}", res);
                         break;
                     } else {
                         log::info!("Exiting normally.");
@@ -188,6 +194,7 @@ pub async fn main_connect(opt: ConnectOpt) -> anyhow::Result<()> {
                     line.clear();
                 }
             }
+            smol::Timer::after(Duration::from_secs(1)).await;
         }
     }
 
@@ -266,11 +273,28 @@ pub async fn main_connect(opt: ConnectOpt) -> anyhow::Result<()> {
             .await
             .context("cannot accept socks5")?;
         let tunnel_manager = tunnel_manager.clone();
-        smolscale::spawn(async move { handle_socks5(s5client, &tunnel_manager, exclude_prc).await })
+        if let Ok(_ticket) = acquire_fd().await {
+            smolscale::spawn(async move {
+                let _ticket = _ticket;
+                handle_socks5(s5client, &tunnel_manager, exclude_prc).await
+            })
             .detach()
+        }
     }
 }
 
+/// Kills geph at a particular port.
+async fn kill_existing_geph(stats_addr: SocketAddr) -> anyhow::Result<()> {
+    let conn = TcpStream::connect(
+        stats_addr.tap_mut(|addr| addr.set_ip(Ipv4Addr::new(127, 0, 0, 1).into())),
+    )
+    .await?;
+    async_h1::connect(conn, Request::new(Method::Get, "/kill"))
+        .await
+        .map_err(|e| e.into_inner())?;
+    Ok(())
+}
+
 /// Returns whether or not we're in China.
 #[cached::proc_macro::cached(result = true)]
 async fn test_china() -> http_types::Result<bool> {
@@ -320,18 +344,22 @@ async fn port_forwarder(tunnel_manager: TunnelManager, desc: String) {
         .expect("could not listen for port forwarding");
     loop {
         let (conn, _) = listener.accept().await.unwrap();
-        let tunnel_manager = tunnel_manager.clone();
-        let remote_addr = exploded[1].to_owned();
-        smolscale::spawn(async move {
-            let remote = tunnel_manager.connect(&remote_addr).await.ok()?;
-            smol::future::race(
-                smol::io::copy(remote.clone(), conn.clone()),
-                smol::io::copy(conn, remote),
-            )
-            .await
-            .ok()
-        })
-        .detach();
+        let _ticket = acquire_fd().await;
+        if let Ok(_ticket) = _ticket {
+            let tunnel_manager = tunnel_manager.clone();
+            let remote_addr = exploded[1].to_owned();
+            smolscale::spawn(async move {
+                let _ticket = _ticket;
+                let remote = tunnel_manager.connect(&remote_addr).await.ok()?;
+                smol::future::race(
+                    smol::io::copy(remote.clone(), conn.clone()),
+                    smol::io::copy(conn, remote),
+                )
+                .await
+                .ok()
+            })
+            .detach();
+        }
     }
 }
 
